# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""JS/CSS Webpack bundles for theme."""

from invenio_assets.webpack import WebpackThemeBundle

theme = WebpackThemeBundle(
    __name__,
    'assets',
    default='semantic-ui',
    themes={
        'semantic-ui': dict(
            entry={
                'invenio-app-rdm-landing-page':
                    './js/invenio_app_rdm/landing_page/index.js',
                'invenio-app-rdm-landing-page-theme':
                    './js/invenio_app_rdm/landing_page/theme.js',
                'invenio-app-rdm-deposit':
                    './js/invenio_app_rdm/deposit/index.js',
                'invenio-app-rdm-search':
                    './js/invenio_app_rdm/search/index.js',
                'invenio-app-rdm-user-uploads':
                './js/invenio_app_rdm/user_dashboard/uploads.js',
                'invenio-app-rdm-user-communities':
                './js/invenio_app_rdm/user_dashboard/communities.js',
                'invenio-app-rdm-user-requests':
                './js/invenio_app_rdm/user_dashboard/requests.js',
                'base-theme-rdm': './js/invenio_app_rdm/theme.js',
            },
            dependencies={
                '@babel/runtime': '^7.9.0',
                '@ckeditor/ckeditor5-build-classic': '^16.0.0',
                '@ckeditor/ckeditor5-react': '^2.1.0',
                'formik': '^2.1.0',
                "i18next": "^20.3.0",
                "i18next-browser-languagedetector": "^6.1.0",
                'luxon': '^1.23.0',
                'path': '^0.12.7',
                'prop-types': '^15.7.2',
                'react-copy-to-clipboard': '^5.0.0',
                'react-dnd': '^11.1.0',
                'react-dnd-html5-backend': '^11.1.0',
                'react-dropzone': "^11.0.0",
                "react-i18next": "^11.11.0",
                'react-invenio-deposit': '^0.19.0',
                'react-invenio-forms': '^0.10.0',
                'react-searchkit': '^2.0.0',
                'yup': '^0.32.0',
            },
            aliases={
                # Define Semantic-UI theme configuration needed by
                # Invenio-Theme in order to build Semantic UI (in theme.js
                # entry point). theme.config itself is provided by
                # cookiecutter-invenio-rdm.
                '../../theme.config$': 'less/theme.config',
                '../../less/site': 'less/site',
                '../../less': 'less',
                'themes/rdm': 'less/invenio_app_rdm/theme',
                '@less/invenio_app_rdm': 'less/invenio_app_rdm',
                '@translations/invenio_app_rdm':
                    'translations/invenio_app_rdm',
            }
        ),
    }
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2020 CERN.
# Copyright (C) 2019-2020 Northwestern University.
# Copyright (C)      2021 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Routes for general pages provided by Invenio-App-RDM."""

from flask import Blueprint, current_app, render_template
from flask_babelex import get_locale
from flask_babelex import lazy_gettext as _
from flask_menu import current_menu


#
# Registration
#
def create_blueprint(app):
    """Blueprint for the routes and resources provided by Invenio-App-RDM."""
    routes = app.config.get("APP_RDM_ROUTES")

    blueprint = Blueprint(
        "invenio_app_rdm",
        __name__,
        template_folder="templates",
        static_folder="static",
    )

    blueprint.add_url_rule(routes["index"], view_func=index)
    blueprint.add_url_rule(routes["help_search"], view_func=help_search)

    @blueprint.before_app_first_request
    def init_menu():
        """Initialize menu before first request."""
        current_menu.submenu('actions.deposit').register(
            'invenio_app_rdm_users.uploads',
            _('My dashboard'),
            order=1
        )

        current_menu.submenu('plus.deposit').register(
            'invenio_app_rdm_records.deposit_create',
            _('New upload'),
            order=1,
        )

        current_menu.submenu("notifications.requests").register(
            "invenio_app_rdm_users.requests",
            order=1,
        )

    return blueprint


#
# Views
#
def index():
    """Frontpage."""
    return render_template(
        current_app.config['THEME_FRONTPAGE_TEMPLATE'],
    )


def help_search():
    """Search help guide."""
    # Default to rendering english page if locale page not found.
    locale = get_locale()
    return render_template([
        f"invenio_app_rdm/help/search.{locale}.html",
        "invenio_app_rdm/help/search.en.html",
    ])



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019-2022 CERN.
# Copyright (C) 2019-2022 Northwestern University.
# Copyright (C)      2022 TU Wien.
#
# Invenio App RDM is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Request views module."""

from flask import g, render_template, url_for
from flask_login import current_user, login_required
from invenio_communities.members.services.request import CommunityInvitation
from invenio_communities.views.decorators import pass_community
from invenio_rdm_records.proxies import current_rdm_records_service
from invenio_rdm_records.requests import CommunitySubmission
from invenio_rdm_records.resources.serializers import UIJSONSerializer
from invenio_records_resources.services.errors import PermissionDeniedError
from invenio_requests.customizations import AcceptAction
from invenio_requests.resolvers.registry import ResolverRegistry
from invenio_requests.views.decorators import pass_request
from invenio_users_resources.proxies import current_user_resources
from sqlalchemy.orm.exc import NoResultFound

from invenio_app_rdm.records_ui.views.decorators import draft_files_service, \
    files_service


def _resolve_topic_draft(request):
    """Resolve the record in the topic when it is a draft."""
    user_owns_request = \
        str(request["expanded"]["created_by"]["id"]) == str(current_user.id)

    if request["is_closed"] and not user_owns_request:
        return dict(permissions={}, record_ui=None)

    recid = ResolverRegistry.resolve_entity_proxy(
        request["topic"]
    )._parse_ref_dict_id()
    try:
        record = current_rdm_records_service.read_draft(
            g.identity, recid, expand=True
        )
        record_ui = UIJSONSerializer().serialize_object_to_dict(
            record.to_dict()
        )
        permissions = record.has_permissions_to(
            [
                "edit",
                "new_version",
                "manage",
                "update_draft",
                "read_files",
                "review",
            ]
        )
        return dict(permissions=permissions, record_ui=record_ui)
    except NoResultFound:
        # record tab not displayed when the record is not found
        # the request is probably not open anymore
        pass

    return dict(permissions={}, record_ui=None)


def _resolve_record_or_draft_files(record):
    """Resolve the record's or draft's files."""
    if record and record["files"]["enabled"]:
        record_pid = record["id"]
        try:
            files = draft_files_service().list_files(
                id_=record_pid, identity=g.identity
            )
        except NoResultFound:
            files = files_service().list_files(
                id_=record_pid, identity=g.identity
            )
        return files.to_dict()
    return None


@login_required
@pass_request(expand=True)
def user_dashboard_request_view(request, **kwargs):
    """User dashboard request details view."""
    avatar = current_user_resources.users_service.links_item_tpl.expand(
        current_user
    )["avatar"]

    request_type = request["type"]

    is_draft_submission = request_type == CommunitySubmission.type_id
    is_invitation = request_type == CommunityInvitation.type_id
    request_is_accepted = request["status"] == AcceptAction.status_to

    if is_draft_submission:
        topic = _resolve_topic_draft(request)
        record = topic['record_ui']
        files = _resolve_record_or_draft_files(record)
        return render_template(
            "invenio_requests/community-submission/index.html",
            base_template="invenio_app_rdm/users/base.html",
            user_avatar=avatar,
            request=request.to_dict(),
            record=record,
            permissions=topic["permissions"],
            is_preview=True,
            draft_is_accepted=request_is_accepted,
            files=files,
        )

    elif is_invitation:
        return render_template(
            "invenio_requests/community-invitation/user_dashboard.html",
            base_template="invenio_app_rdm/users/base.html",
            user_avatar=avatar,
            request=request.to_dict(),
            invitation_accepted=request_is_accepted,
        )


@login_required
@pass_request(expand=True)
@pass_community
def community_dashboard_request_view(request, community, **kwargs):
    """Community dashboard requests details view."""
    request_type = request["type"]

    is_draft_submission = request_type == CommunitySubmission.type_id
    is_invitation = request_type == CommunityInvitation.type_id
    request_is_accepted = request["status"] == AcceptAction.status_to
    if is_draft_submission:
        permissions = community.has_permissions_to(
            ["update", "read", "search_requests", "search_invites"]
        )

        topic = _resolve_topic_draft(request)
        permissions.update(topic["permissions"])
        record = topic['record_ui']
        files = _resolve_record_or_draft_files(record)
        return render_template(
            "invenio_requests/community-submission/index.html",
            base_template="invenio_communities/details/base.html",
            request=request.to_dict(),
            record=record,
            community=community.to_dict(),
            permissions=permissions,
            is_preview=True,
            draft_is_accepted=request_is_accepted,
            files=files,
        )

    elif is_invitation:
        permissions = community.has_permissions_to(
            ["update", "read", "search_requests", "search_invites"]
        )
        if not permissions["can_search_invites"]:
            raise PermissionDeniedError()

        return render_template(
            "invenio_requests/community-invitation/community_dashboard.html",
            base_template="invenio_communities/details/members/base.html",
            request=request.to_dict(),
            community=community.to_dict(),
            permissions=permissions,
            invitation_accepted=request_is_accepted,
        )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 CERN.
#
# Invenio-App-RDM is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Configuration helper for React-SearchKit."""

from functools import partial

from flask import current_app
from invenio_search_ui.searchconfig import search_app_config


def search_app_context():
    """Search app context processor."""
    return {
        'search_app_rdm_user_uploads_config': partial(
            search_app_config,
            'RDM_SEARCH_DRAFTS',
            current_app.config['RDM_FACETS'],
            current_app.config['RDM_SORT_OPTIONS'],
            '/api/user/records',
            {"Accept": "application/vnd.inveniordm.v1+json"},
        ),
        'search_app_rdm_user_communities_config': partial(
            search_app_config,
            'RDM_SEARCH_USER_COMMUNITIES',
            current_app.config['COMMUNITIES_FACETS'],
            current_app.config['RDM_SORT_OPTIONS'],
            '/api/user/communities',
            {"Accept": "application/json"}
        ),
        'search_app_rdm_user_requests_config': partial(
            search_app_config,
            'RDM_SEARCH_USER_REQUESTS',
            current_app.config['REQUESTS_FACETS'],
            current_app.config['RDM_SORT_OPTIONS'],
            '/api/user/requests',
            {"Accept": "application/json"},
            initial_filters=[["is_open", "true"]],
            hidden_params=[["expand", "1"]]
        )
    }
